@@ -19,8 +19,7 @@ def fdq_train(experiment: fdqExperiment) -> None:
     device_type = "cuda" if experiment.device == torch.device("cuda") else "cpu"
 
     for epoch in range(experiment.start_epoch, experiment.nb_epochs):
-        experiment.current_epoch = epoch
-        iprint(f"\nEpoch: {epoch + 1} / {experiment.nb_epochs}")
+        experiment.on_epoch_start(epoch=epoch)
 
         train_loss_sum = 0.0
         val_loss_sum = 0.0
@@ -36,18 +35,13 @@ def fdq_train(experiment: fdqExperiment) -> None:
 
             with torch.autocast(device_type=device_type, enabled=experiment.useAMP):
                 output = model(inputs)
-                loss_tensor = (
-                    experiment.losses["cross_ent"](output, targets)
-                    / experiment.gradacc_iter
-                )
+                loss_tensor = experiment.losses["cross_ent"](output, targets) / experiment.gradacc_iter
                 if experiment.useAMP and experiment.scaler is not None:
                     experiment.scaler.scale(loss_tensor).backward()
                 else:
                     loss_tensor.backward()
 
-            experiment.update_gradients(
-                b_idx=nb_batch, loader_name="MNIST", model_name="simpleNet"
-            )
+            experiment.update_gradients(b_idx=nb_batch, loader_name="MNIST", model_name="simpleNet")
 
             train_loss_sum += loss_tensor.detach().item()
 
@@ -91,17 +85,14 @@ def fdq_train(experiment: fdqExperiment) -> None:
             "dataformats": "CHW",
         }
 
-        captions = [
-            f"Predicted: {preds[idx].item()}, True: {targets[idx].item()}"
-            for idx in range(len(preds))
-        ]
+        captions = [f"Predicted: {preds[idx].item()}, True: {targets[idx].item()}" for idx in range(len(preds))]
         imgs_wandb = {
             "name": "inputs",
             "data": inputs[:max_log_size],
             "captions": captions[:max_log_size],
         }
 
-        experiment.finalize_epoch(
+        experiment.on_epoch_end(
             log_images_wandb=imgs_wandb,
             log_images_tensorboard=imgs_tb,
             log_text_tensorboard=log_txt,
