@@ -5,7 +5,7 @@ from urllib.request import urlretrieve
 import torch
 from torch.utils.data.distributed import DistributedSampler
 from torch.utils.data import DataLoader
-from torchvision.transforms import v2 as transforms
+from torchvision.transforms import v2
 import numpy as np
 from PIL import Image
 from tqdm import tqdm
@@ -41,7 +41,7 @@ class OxfordPetDataset(torch.utils.data.Dataset):
         self.root = root
         self.mode = mode
         self.binary = binary
-        self.to_tensor = transforms.ToTensor()
+        self.to_tensor = v2.Compose([v2.ToImage(), v2.ToDtype(torch.float32, scale=True)])
         self.transform_img = transform_image
         self.transform_mask = transform_mask
 
@@ -65,18 +65,12 @@ class OxfordPetDataset(torch.utils.data.Dataset):
 
         if self.binary:
             mask = torch.where(mask == 2.0, torch.tensor(0.0, dtype=mask.dtype), mask)
-            mask = torch.where(
-                (mask == 1.0) | (mask == 3.0), torch.tensor(1.0, dtype=mask.dtype), mask
-            )
+            mask = torch.where((mask == 1.0) | (mask == 3.0), torch.tensor(1.0, dtype=mask.dtype), mask)
             # add channel dimension
             mask = mask.unsqueeze(0)
         else:
             # one hot encoding
-            mask = (
-                torch.nn.functional.one_hot((mask - 1).long(), num_classes=3)
-                .permute(2, 0, 1)
-                .float()
-            )
+            mask = torch.nn.functional.one_hot((mask - 1).long(), num_classes=3).permute(2, 0, 1).float()
 
         if self.transform_img is not None:
             image = self.transform_img(image)
@@ -85,7 +79,7 @@ class OxfordPetDataset(torch.utils.data.Dataset):
 
         return {"image": image, "mask": mask}
 
-    def _read_split(self):
+    def _read_split(self) -> list[str]:
         split_filename = "test.txt" if self.mode == "test" else "trainval.txt"
         split_filepath = os.path.join(self.root, "annotations", split_filename)
         with open(split_filepath, encoding="utf8") as f:
@@ -98,7 +92,7 @@ class OxfordPetDataset(torch.utils.data.Dataset):
         return filenames
 
     @staticmethod
-    def download(root):
+    def download(root: str) -> None:
         # load images
         filepath = os.path.join(root, "images.tar.gz")
         download_url(
@@ -116,7 +110,7 @@ class OxfordPetDataset(torch.utils.data.Dataset):
         extract_archive(filepath)
 
 
-def create_datasets(experiment, args=None):
+def create_datasets(experiment, args=None) -> dict:
     """Creates and returns data loaders and dataset statistics for the Oxford Pet dataset based on the experiment configuration."""
     pin_mem = True if experiment.is_cuda else args.get("pin_memory", False)
     drop_last = args.get("drop_last", True)
@@ -183,6 +177,7 @@ def create_datasets(experiment, args=None):
         pin_memory=pin_mem,
         drop_last=drop_last,
         sampler=train_sampler,
+        prefetch_factor=args.prefetch_factor,
     )
 
     test_loader = DataLoader(
@@ -193,6 +188,7 @@ def create_datasets(experiment, args=None):
         pin_memory=pin_mem,
         drop_last=drop_last,
         sampler=test_sampler,
+        prefetch_factor=args.prefetch_factor,
     )
 
     val_loader = DataLoader(
@@ -203,8 +199,11 @@ def create_datasets(experiment, args=None):
         pin_memory=pin_mem,
         drop_last=drop_last,
         sampler=val_sampler,
+        prefetch_factor=args.prefetch_factor,
     )
 
+    # No mandatory structure here, instead all values
+    # that are returned can be accessed in the training loop.
     return {
         "train_data_loader": train_loader,
         "val_data_loader": val_loader,
@@ -224,13 +223,13 @@ def create_datasets(experiment, args=None):
 class TqdmUpTo(tqdm):
     """A tqdm progress bar subclass that provides an 'update_to' method for use with urlretrieve reporthook."""
 
-    def update_to(self, b=1, bsize=1, tsize=None):
+    def update_to(self, b: int = 1, bsize: int = 1, tsize: int | None = None) -> None:
         if tsize is not None:
             self.total = tsize
         self.update(b * bsize - self.n)
 
 
-def download_url(url, filepath):
+def download_url(url: str, filepath: str) -> None:
     """Download a file from a URL to the specified filepath, showing a progress bar."""
     directory = os.path.dirname(os.path.abspath(filepath))
     os.makedirs(directory, exist_ok=True)
@@ -248,7 +247,7 @@ def download_url(url, filepath):
         t.total = t.n
 
 
-def extract_archive(filepath):
+def extract_archive(filepath: str) -> None:
     """Extracts the archive at the given filepath to its containing directory if not already extracted."""
     extract_dir = os.path.dirname(os.path.abspath(filepath))
     dst_dir = os.path.splitext(filepath)[0]
