@@ -15,42 +15,30 @@ def fdq_train(experiment: fdqExperiment) -> None:
 
     data = experiment.data["OXPET"]
     model = experiment.models["ccUNET"]
-    device_type = "cuda" if experiment.device == torch.device("cuda") else "cpu"
 
     for epoch in range(experiment.start_epoch, experiment.nb_epochs):
-        experiment.current_epoch = epoch
-        iprint(f"\nEpoch: {epoch + 1} / {experiment.nb_epochs}")
+        experiment.on_epoch_start(epoch=epoch)
 
         train_loss_sum = 0.0
         val_loss_sum = 0.0
         model.train()
         pbar = startProgBar(data.n_train_samples, "training...")
 
-        if experiment.is_distributed():
-            # necessary to make shuffling work properly
-            data.train_sampler.set_epoch(epoch)
-            data.val_sampler.set_epoch(epoch)
-
         for nb_tbatch, batch in enumerate(data.train_data_loader):
             pbar.update(nb_tbatch * len(batch["image"]))
 
             inputs = batch["image"].to(experiment.device).type(torch.float32)
             targets = batch["mask"].to(experiment.device).type(torch.float32)
 
-            with torch.autocast(device_type=device_type, enabled=experiment.useAMP):
+            with torch.autocast(device_type=experiment.device.type, enabled=experiment.useAMP):
                 output = model(inputs)
-                train_loss_tensor = (
-                    experiment.losses["cross_ent"](output, targets)
-                    / experiment.gradacc_iter
-                )
+                train_loss_tensor = experiment.losses["cross_ent"](output, targets) / experiment.gradacc_iter
                 if experiment.useAMP and experiment.scaler is not None:
                     experiment.scaler.scale(train_loss_tensor).backward()
                 else:
                     train_loss_tensor.backward()
 
-            experiment.update_gradients(
-                b_idx=nb_tbatch, loader_name="OXPET", model_name="ccUNET"
-            )
+            experiment.update_gradients(b_idx=nb_tbatch, loader_name="OXPET", model_name="ccUNET")
 
             train_loss_sum += train_loss_tensor.detach().item()
 
@@ -81,7 +69,7 @@ def fdq_train(experiment: fdqExperiment) -> None:
             {"name": "target", "data": targets, "dataformats": "NCHW"},
         ]
 
-        experiment.finalize_epoch(log_images_wandb=img, log_images_tensorboard=img)
+        experiment.on_epoch_end(log_images_wandb=img, log_images_tensorboard=img)
 
         if experiment.check_early_stop():
             break
