@@ -43,7 +43,7 @@ def user_set_dtype(example: torch.Tensor) -> torch.Tensor:
     print("Example data type:", example.dtype)
     print("Example shape:", example.shape)
     sel_mode: int = getIntInput(
-        "Define model tracing input dtype:\n  1) float32\n  2) float16\n  3) int8\n  4) float64\n",
+        "Define example input dtype:\n  1) float32\n  2) float16\n  3) int8\n  4) float64\n",
         drange=[1, 4],
     )
     if sel_mode == 1:
@@ -62,7 +62,7 @@ def get_example_tensor(experiment: Any) -> torch.Tensor:
     sources = list(experiment.data.keys())
     idx: int = (
         getIntInput(
-            f"Select data source for tracing sample shape: {[f'{i + 1}) {src}' for i, src in enumerate(sources)]}",
+            f"\nSelect data sample source: {[f'{i + 1}) {src}' for i, src in enumerate(sources)]}",
             drange=[1, len(sources)],
         )
         - 1
@@ -199,7 +199,7 @@ def jit_trace_model(
     return jit_model, config
 
 
-def compile_model(
+def optimize_model(
     config: dict[str, Any],
     experiment: Any,
     example: torch.Tensor,
@@ -207,17 +207,16 @@ def compile_model(
     model_name: str,
 ) -> None:
     """Compile, optionally JIT trace/script, and optimize a model using Torch-TensorRT, with interactive configuration and testing."""
+    iprint("\n-----------------------------------------------------------")
+    iprint("Optimize model")
+    iprint("-----------------------------------------------------------\n")
     import torch_tensorrt
     from torch_tensorrt import Input
 
     try:
-        jit_model, config = jit_trace_model(
-            experiment, config, model, model_name, example
-        )
-
+        jit_model, config = jit_trace_model(experiment, config, model, model_name, example)
     except (RuntimeError, TypeError, ValueError) as e:
-        wprint("Failed to JIT Trace model!")
-        print(e)
+        raise RuntimeError(f"Failed to JIT process model (trace/script): {e}") from e
 
     inputs = [
         Input(
@@ -227,21 +226,18 @@ def compile_model(
         )
     ]
 
-    if getYesNoInput("Compile model? (y/n)\n"):
+    if getYesNoInput("Torch.compile() model? (y/n)\n"):
         if config["jit_traced"] or config["jit_scripted"]:
+            iprint("Using JIT model for torch.compile()")
             inter_rep = "torchscript"
         else:
             inter_rep_choice = getIntInput(
-                "Select intermediate representation:\n"
-                "  1) default: Let Torch-TensorRT decide\n"
-                "  2) ts: TorchScript\n",
+                "Select intermediate representation:\n  1) default: Let Torch-TensorRT decide\n  2) ts: TorchScript\n",
                 drange=[1, 2],
             )
             inter_rep = "default" if inter_rep_choice == 1 else "ts"
 
-        truncate_double = getYesNoInput(
-            "Truncate long and double? (y/n), default = y\n"
-        )
+        truncate_double = getYesNoInput("Truncate long and double? (y/n), default = y\n")
 
         enabled_precisions = set()
         if getYesNoInput("Enable float32 precision? (y/n)"):
@@ -280,35 +276,112 @@ def compile_model(
             run_test(experiment, example, model, optimized_model, config)
 
         if getYesNoInput("Save optimized model? (y/n)"):
-            save_path = os.path.join(
-                experiment.results_dir, f"{model_name}_optimized.ts"
-            )
+            save_path = os.path.join(experiment.results_dir, f"{model_name}_optimized.ts")
             torch.save(optimized_model, save_path)
             iprint(f"Optimized model saved to {save_path}")
 
 
+def export_onnx_model(
+    config: dict[str, Any],
+    experiment: Any,
+    example: torch.Tensor,
+    model: torch.nn.Module,
+    model_name: str,
+) -> None:
+    """Export the given model to ONNX format, optionally using Dynamo, optimizing, and saving the exported model.
+
+    Parameters
+    ----------
+    config : dict[str, Any]
+        Configuration dictionary for export options.
+    experiment : Any
+        Experiment object containing model and data information.
+    example : torch.Tensor
+        Example input tensor for tracing the model.
+    model : torch.nn.Module
+        The PyTorch model to export.
+    model_name : str
+        Name of the model for saving the exported file.
+
+    Returns:
+    -------
+    None
+    """
+    iprint("\n-----------------------------------------------------------")
+    iprint("Export ONNX Model")
+    iprint("-----------------------------------------------------------\n")
+
+    save_path = os.path.join(experiment.results_dir, f"{model_name}.onnx")
+    model_saved = False
+
+    use_dynamo = getYesNoInput("Use dynamo for ONNX export? (y/n), default = n\n")
+    if use_dynamo:
+        save_path = save_path.replace(".onnx", "_dynamo.onnx")
+        onnx_program = torch.onnx.export(
+            model,
+            example,
+            # save_path,
+            export_params=True,
+            # opset_version=12,
+            input_names=["input"],
+            output_names=["output"],
+            dynamo=use_dynamo,
+        )
+    else:
+        save_path = save_path.replace(".onnx", "_torchscript.onnx")
+        torch.onnx.export(
+            model,
+            example,
+            save_path,
+            export_params=True,
+            opset_version=12,
+            input_names=["input"],
+            output_names=["output"],
+        )
+        model_saved = True
+
+    iprint("ONNX model created!")
+
+    if use_dynamo:
+        if getYesNoInput("Optimize model (y/n)"):
+            onnx_program.optimize()
+            save_path = save_path.replace(".onnx", "_optimized.onnx")
+
+        if getYesNoInput("Save ONNX model? (y/n)"):
+            onnx_program.save(save_path)
+            model_saved = True
+
+    if model_saved:
+        file_size_mb = os.path.getsize(save_path) / (1024 * 1024)
+        iprint(f"ONNX model exported to {save_path}")
+        iprint(f"File size: {file_size_mb:.2f} MB")
+        iprint("You can use 'https://netron.app/' to visualize the exported model.")
+
+
 def dump_model(experiment: Any) -> None:
     """Interactively dumps, traces, scripts, compiles, tests, and saves a model from the given experiment."""
     iprint("\n-----------------------------------------------------------")
     iprint("Dump model")
     iprint("-----------------------------------------------------------\n")
+
     if experiment.is_distributed():
-        raise ValueError(
-            "ERROR: Cannot dump with world size > 1; please run in single process mode."
-        )
-    # dumping requires a sample input to trace the model
-    # -> set exp t train mode so that train loader is created.
-    experiment.mode.train()
+        raise ValueError("ERROR: Cannot dump with world size > 1; please run in single process mode.")
+
     experiment.setupData()
     experiment.init_models(instantiate=False)
 
     select_experiment(experiment)
     model_name, model = select_model(experiment)
-    iprint(f"Compiling {model_name}...")
+
+    iprint(f"Processing {model_name}...")
 
     while True:
-        example = get_example_tensor(experiment)
+        dump_mode = getIntInput(
+            "Select Operation:\n  1) Optimize model (JIT trace / Script, and torch.compile)\n  2) ONNX export\n",
+            drange=[1, 2],
+        )
 
+        example = get_example_tensor(experiment)
         config: dict[str, Any] = {
             "jit_traced": False,
             "jit_scripted": False,
@@ -316,15 +389,19 @@ def dump_model(experiment: Any) -> None:
             "input dtype": example.dtype,
         }
 
-        try:
-            compile_model(config, experiment, example, model, model_name)
+        if dump_mode == 1:
+            try:
+                optimize_model(config, experiment, example, model, model_name)
+            except (RuntimeError, TypeError, ValueError) as e:
+                wprint("Failed to optimize model!")
+                print(e)
 
-        except (RuntimeError, TypeError, ValueError) as e:
-            wprint("Failed to compile model!")
-            print(e)
-            if getYesNoInput("Try again? (y/n)"):
-                continue
-            break
+        elif dump_mode == 2:
+            try:
+                export_onnx_model(config, experiment, example, model, model_name)
+            except (RuntimeError, TypeError, ValueError) as e:
+                wprint("Failed to export ONNX model!")
+                print(e)
 
-        if not getYesNoInput("Dump another model? (y/n)"):
+        if not getYesNoInput("\nProcess another model? (y/n)"):
             break
