@@ -24,7 +24,6 @@ from fdq.ui_functions import (
 )
 from fdq.testing import find_model_path
 from fdq.transformers import get_transformers
-from fdq.dump import dump_model
 from fdq.misc import (
     remove_file,
     store_processing_infos,
@@ -35,6 +34,7 @@ from fdq.misc import (
     save_wandb,
     load_conf_file,
 )
+from fdq.dataset_caching import cache_datasets_ddp_handler
 
 
 class fdqExperiment:
@@ -55,19 +55,16 @@ class fdqExperiment:
         self.exp_def = load_conf_file(self.experiment_file_path)
         self.globals = self.exp_def.globals
         self.project: str = self.exp_def.globals.project.replace(" ", "_")
-        self.experimentName: str = self.experiment_file_path.split("/")[-1].split(
-            ".json"
-        )[0]
+        self.experimentName: str = self.experiment_file_path.split("/")[-1].split(".json")[0]
         self.funky_name: str | None = None
         self.checkpoint_frequency: int = self.exp_def.store.checkpoint_frequency
         self.mode: FCQmode = FCQmode()
         self.creation_time: datetime = datetime.now()
+        self.current_ep_start_time: datetime | None = None
         self.finish_time: datetime | None = None
-        self.run_time: timedelta | None = None
+        self.total_run_time: timedelta | None = None
         self.run_info: dict[str, Any] = {}
-        self.gradacc_iter: int = self.exp_def.train.args.get(
-            "accumulate_grad_batches", default=1
-        )
+        self.gradacc_iter: int = self.exp_def.train.args.get("accumulate_grad_batches", default=1)
         self.useAMP: bool = bool(self.exp_def.train.args.use_AMP)
         self.nb_epochs: int = self.exp_def.train.args.epochs
         self.current_epoch: int = 0
@@ -111,9 +108,7 @@ class fdqExperiment:
         if isinstance(slurm_job_id, str) and slurm_job_id.isdigit():
             self.is_slurm: bool = True
             self.slurm_job_id: str = slurm_job_id
-            self.scratch_data_path: str | None = self.exp_def.get(
-                "slurm_cluster", {}
-            ).get("scratch_data_path")
+            self.scratch_data_path: str | None = self.exp_def.get("slurm_cluster", {}).get("scratch_data_path")
         else:
             self.is_slurm = False
             self.slurm_job_id = None
@@ -124,13 +119,9 @@ class fdqExperiment:
         if not self.inargs.train_model:
             self.world_size = 1
         else:
-            self.world_size: int = self.exp_def.get("slurm_cluster", {}).get(
-                "world_size", 1
-            )
+            self.world_size: int = self.exp_def.get("slurm_cluster", {}).get("world_size", 1)
         self.master_port: int = self.exp_def.get("slurm_cluster", {}).get("master_port")
-        self.ddp_rdvz_path: int = self.exp_def.get("slurm_cluster", {}).get(
-            "ddp_rdvz_path", "/scratch/"
-        )
+        self.ddp_rdvz_path: int = self.exp_def.get("slurm_cluster", {}).get("ddp_rdvz_path", "/scratch/")
         self.init_distributed_mode()
 
         self.previous_slurm_job_id: str | None = None
@@ -138,9 +129,7 @@ class fdqExperiment:
             torch.cuda.empty_cache()
             self.device: torch.device = torch.device("cuda", self.rank)
             self.is_cuda: bool = True
-            iprint(
-                f"CUDA available: {torch.cuda.is_available()}. NB devices: {torch.cuda.device_count()}"
-            )
+            iprint(f"CUDA available: {torch.cuda.is_available()}. NB devices: {torch.cuda.device_count()}")
         else:
             wprint("NO CUDA available - CPU mode")
             self.device = torch.device("cpu")
@@ -162,9 +151,7 @@ class fdqExperiment:
 
             if self.is_slurm:
                 folder_name += f"__{self.slurm_job_id}"
-                res_base_path = self.exp_def.get("slurm_cluster", {}).get(
-                    "scratch_results_path"
-                )
+                res_base_path = self.exp_def.get("slurm_cluster", {}).get("scratch_results_path")
                 if res_base_path is None:
                     raise ValueError("Error, scratch_results_path was not defined.")
 
@@ -173,9 +160,7 @@ class fdqExperiment:
                 if res_base_path is None:
                     raise ValueError("Error, result path was not defined.")
 
-            self._results_dir = os.path.join(
-                res_base_path, self.project, self.experimentName, folder_name
-            )
+            self._results_dir = os.path.join(res_base_path, self.project, self.experimentName, folder_name)
 
             if not os.path.exists(self._results_dir):
                 os.makedirs(self._results_dir)
@@ -188,9 +173,7 @@ class fdqExperiment:
             return None
 
         if self._results_output_dir is None:
-            self._results_output_dir = os.path.join(
-                self.results_dir, "training_outputs"
-            )
+            self._results_output_dir = os.path.join(self.results_dir, "training_outputs")
             if not os.path.exists(self._results_output_dir):
                 os.makedirs(self._results_output_dir)
         return self._results_output_dir
@@ -243,10 +226,21 @@ class fdqExperiment:
             return True
         return self.rank == 0
 
+    def is_child_process(self) -> bool:
+        """Check if the current process is a child process in a distributed setup."""
+        if not self.is_distributed():
+            return False
+        return self.rank > 0
+
     def is_distributed(self) -> bool:
         """Check if the current setup is distributed."""
         return self.world_size > 1
 
+    def dist_barrier(self) -> None:
+        """Barrier for distributed training."""
+        if self.is_distributed():
+            torch.distributed.barrier()
+
     def init_distributed_mode(self):
         # if "SLURM_PROCID" not in os.environ or os.environ["SLURM_JOB_NAME"] == "bash":
         #     return
@@ -260,9 +254,7 @@ class fdqExperiment:
 
         dist_backend = "nccl"
         # dist_url = "env://"
-        rdvz_location = (
-            f"file://{self.ddp_rdvz_path}ddp_rendezvous_{self.experimentName}"
-        )
+        rdvz_location = f"file://{self.ddp_rdvz_path}ddp_rendezvous_{self.experimentName}"
 
         iprint("Initializing distributed mode.")
         iprint(f"world size {self.world_size}, rank: {self.rank}")
@@ -303,9 +295,7 @@ class fdqExperiment:
         if parent_dir not in sys.path:
             sys.path.append(parent_dir)
 
-    def import_class(
-        self, file_path: str | None = None, module_name: str | None = None
-    ) -> Any:
+    def import_class(self, file_path: str | None = None, module_name: str | None = None) -> Any:
         if module_name is None:
             if file_path is None or not file_path.endswith(".py"):
                 raise ValueError(
@@ -322,9 +312,7 @@ class fdqExperiment:
         class_name: str | None = None,
     ) -> Any:
         if class_path is None and file_path is None:
-            raise ValueError(
-                f"Error, class_path or file_path must be defined. Got: {class_path}, {file_path}"
-            )
+            raise ValueError(f"Error, class_path or file_path must be defined. Got: {class_path}, {file_path}")
 
         if class_path is not None:
             if "." not in class_path:
@@ -340,16 +328,12 @@ class fdqExperiment:
                     f"Error, path must be a string with the module name and class name separated by a dot. Got: {file_path}"
                 )
             if class_name is None:
-                raise ValueError(
-                    f"Error, class_name must be defined if file_path is used. Got: {class_name}"
-                )
+                raise ValueError(f"Error, class_name must be defined if file_path is used. Got: {class_name}")
             self.add_module_to_syspath(file_path)
             module_name = os.path.basename(file_path).split(".")[0]
             module = importlib.import_module(module_name)
         else:
-            raise ValueError(
-                f"Error, class_path or file_path must be defined. Got: {class_path}, {file_path}"
-            )
+            raise ValueError(f"Error, class_path or file_path must be defined. Got: {class_path}, {file_path}")
 
         return getattr(module, class_name)
 
@@ -365,72 +349,59 @@ class fdqExperiment:
         for model_name, model_def in self.exp_def.models:
             if model_def.path is not None:
                 if os.path.exists(model_def.path):
-                    cls = self.instantiate_class(
-                        file_path=model_def.path, class_name=model_def.class_name
-                    )
+                    cls = self.instantiate_class(file_path=model_def.path, class_name=model_def.class_name)
                 else:
                     # if path does not exist, it might be a a fdq model specified by the file name only.
                     current_file_path = os.path.abspath(__file__)
-                    networks_dir = os.path.abspath(
-                        os.path.join(os.path.dirname(current_file_path), "../networks/")
-                    )
+                    networks_dir = os.path.abspath(os.path.join(os.path.dirname(current_file_path), "../networks/"))
                     model_path = os.path.join(networks_dir, model_def.path)
-                    cls = self.instantiate_class(
-                        file_path=model_path, class_name=model_def.class_name
-                    )
+                    cls = self.instantiate_class(file_path=model_path, class_name=model_def.class_name)
 
             elif model_def.class_name is not None:
                 # model is an installed pip package
                 cls = self.instantiate_class(class_path=model_def.class_name)
             else:
-                raise ValueError(
-                    f"Error, model {model_name} must have a path or module defined."
-                )
+                raise ValueError(f"Error, model {model_name} must have a path or module defined.")
 
             # load trained model from automatically detected path
             # -> only used for testing or dumping - highest priority
             if self.trained_model_paths.get(model_name) is not None:
-                self.models[model_name] = self.load_model_from_path(
-                    self.trained_model_paths[model_name]
-                )
+                self.models[model_name] = self.load_model_from_path(self.trained_model_paths[model_name])
 
             # load trained model from path defined in exp file
             elif model_def.trained_model_path is not None:
-                self.models[model_name] = self.load_model_from_path(
-                    model_def.trained_model_path
-                )
+                self.models[model_name] = self.load_model_from_path(model_def.trained_model_path)
 
             # or instantiate new model with random weights
             elif instantiate:
-                self.models[model_name] = cls(**model_def.args.to_dict()).to(
-                    self.device
-                )
+                self.models[model_name] = cls(**model_def.args.to_dict()).to(self.device)
                 iprint(
                     f"Model {model_name} instantiated on rank {self.rank}.",
-                    distributed=True,
+                    dist_print=True,
                 )
 
-            if self.is_distributed():
-                self.dist_barrier()
+            if model_name in self.models:
+                if self.is_distributed():
+                    self.dist_barrier()
 
-                self.models[model_name] = DDP(
-                    self.models[model_name].cuda(self.rank),
-                    device_ids=[self.rank],
-                    # find_unused_parameters=True,
-                )
-                iprint(
-                    f"Model {model_name} wrapped in DDP on rank {self.rank}. ",
-                    distributed=True,
-                )
-                self.models_no_ddp[model_name] = self.models[model_name].module
-            else:
-                self.models_no_ddp[model_name] = self.models[model_name]
+                    self.models[model_name] = DDP(
+                        self.models[model_name].cuda(self.rank),
+                        device_ids=[self.rank],
+                        # find_unused_parameters=True,
+                    )
+                    iprint(
+                        f"Model {model_name} wrapped in DDP on rank {self.rank}. ",
+                        dist_print=True,
+                    )
+                    self.models_no_ddp[model_name] = self.models[model_name].module
+                else:
+                    self.models_no_ddp[model_name] = self.models[model_name]
 
-            # frozen model? disable gradient tracking
-            if model_def.freeze:
-                iprint(f"Freezing model {model_name} parameters.")
-                for param in self.models[model_name].parameters():
-                    param.requires_grad = False
+                # frozen model? disable gradient tracking
+                if model_def.freeze:
+                    iprint(f"Freezing model {model_name} parameters.")
+                    for param in self.models[model_name].parameters():
+                        param.requires_grad = False
 
     def load_trained_models(self) -> None:
         """Load trained models, defined by user path or previous trainings.
@@ -443,9 +414,7 @@ class fdqExperiment:
         for model_name, _ in self.exp_def.models:
             if self.mode.test_mode.custom_path:
                 while True:
-                    model_path = input(
-                        f"Enter path to model for '{model_name}' (or 'q' to quit)."
-                    )
+                    model_path = input(f"Enter path to model for '{model_name}' (or 'q' to quit).")
                     if model_path == "q":
                         sys.exit()
                     elif os.path.exists(model_path):
@@ -469,11 +438,17 @@ class fdqExperiment:
         if self.data:
             # data already loaded, skip setup
             return
+
         self.copy_data_to_scratch()
+
         for data_name, data_source in self.exp_def.data.items():
             processor = self.import_class(file_path=data_source.processor)
-            args = self.exp_def.data.get(data_name).args
-            self.data[data_name] = DictToObj(processor.create_datasets(self, args))
+
+            if data_source.caching is None:
+                self.data[data_name] = DictToObj(processor.create_datasets(self, self.exp_def.data.get(data_name).args))
+            else:
+                self.data[data_name] = DictToObj(cache_datasets_ddp_handler(self, processor, data_name, data_source))
+
         self.print_dataset_infos()
 
     def save_current_model(self) -> None:
@@ -501,10 +476,7 @@ class fdqExperiment:
                 torch.save(model, self.last_model_path[model_name])
 
             # new best val loss (default!)
-            if (
-                self.exp_def.store.get("save_best_val_model", False)
-                and self.new_best_val_loss
-            ):
+            if self.exp_def.store.get("save_best_val_model", False) and self.new_best_val_loss:
                 best_model_path = os.path.join(
                     self.results_dir,
                     f"best_val_{model_name}_e{self.current_epoch}.fdqm",
@@ -534,9 +506,7 @@ class fdqExperiment:
             iprint(f"Model: {model_name}")
             nbp = sum(p.numel() for p in model.parameters())
             iprint(f"nb parameters: {nbp / 1e6:.2f}M")
-            iprint(
-                f"Using Float32, This will require {nbp * 4 / 1e9:.3f} GB of memory."
-            )
+            iprint(f"Using Float32, This will require {nbp * 4 / 1e9:.3f} GB of memory.")
             iprint("-----------------------------------------------------------")
             self.processing_log_dict[model_name] = {
                 "nb_parameters": f"{nbp / 1e6:.2f}M",
@@ -562,9 +532,7 @@ class fdqExperiment:
             self.scaler = torch.amp.GradScaler(device=self.device, enabled=True)
 
         if self.inargs.resume_path is not None:
-            iprint(
-                "--------------------------------------------------------------------------"
-            )
+            iprint("-----------------------------------------------------------")
             iprint(f"Loading checkpoint: {self.inargs.resume_path}")
 
             self.load_checkpoint(self.inargs.resume_path)
@@ -617,27 +585,19 @@ class fdqExperiment:
 
             cls = self.instantiate_class(lr_scheduler_module)
 
-            self.lr_schedulers[model_name] = cls(
-                self.optimizers[model_name], **margs.lr_scheduler.args.to_dict()
-            )
+            self.lr_schedulers[model_name] = cls(self.optimizers[model_name], **margs.lr_scheduler.args.to_dict())
 
     def createLosses(self) -> None:
         if self.exp_def.losses is None:
-            wprint(
-                "No losses defined in the experiment file. Losses must be defined in the training loop."
-            )
+            wprint("No losses defined in the experiment file. Losses must be defined in the training loop.")
             return
         for loss_name, largs in self.exp_def.losses:
             if largs.path is not None:
-                cls = self.instantiate_class(
-                    file_path=largs.path, class_name=largs.class_name
-                )
+                cls = self.instantiate_class(file_path=largs.path, class_name=largs.class_name)
             elif largs.class_name is not None:
                 cls = self.instantiate_class(class_path=largs.class_name)
             else:
-                raise ValueError(
-                    f"Error, loss {loss_name} must have a path or class name defined."
-                )
+                raise ValueError(f"Error, loss {loss_name} must have a path or class name defined.")
             if largs.args is not None:
                 self.losses[loss_name] = cls(**largs.args.to_dict())
             else:
@@ -651,9 +611,7 @@ class fdqExperiment:
         try:
             # checkpoint was saved on rank 0
             # so we need to map to the current rank
-            map_location = (
-                {f"cuda:{0}": f"cuda:{self.rank}"} if self.is_distributed() else None
-            )
+            map_location = {f"cuda:{0}": f"cuda:{self.rank}"} if self.is_distributed() else None
             checkpoint = torch.load(path, map_location=map_location)
             self.start_epoch = checkpoint["epoch"]
             self.trainLoss = checkpoint["train_loss"]
@@ -663,9 +621,7 @@ class fdqExperiment:
         except Exception as exc:
             raise ValueError(f"Error loading checkpoint {path}.") from exc
 
-        iprint(
-            f"Loaded checkpoint {self.start_epoch}. Train loss: {self.trainLoss:.4f}, val loss: {self.valLoss:.4f}"
-        )
+        iprint(f"Loaded checkpoint {self.start_epoch}. Train loss: {self.trainLoss:.4f}, val loss: {self.valLoss:.4f}")
 
         if self.start_epoch >= self.nb_epochs - 1:
             raise ValueError(
@@ -676,16 +632,12 @@ class fdqExperiment:
             if model_def.freeze:
                 iprint(f"Skipping loading of frozen model {model_name}.")
                 continue
-            self.models_no_ddp[model_name].load_state_dict(
-                checkpoint["model_state_dict"][model_name]
-            )
+            self.models_no_ddp[model_name].load_state_dict(checkpoint["model_state_dict"][model_name])
 
             if checkpoint["optimizer"] is None:
                 self.optimizers[model_name] = None
             else:
-                self.optimizers[model_name].load_state_dict(
-                    checkpoint["optimizer"][model_name]
-                )
+                self.optimizers[model_name].load_state_dict(checkpoint["optimizer"][model_name])
 
     def save_checkpoint(self) -> None:
         if not self.is_main_process():
@@ -699,9 +651,7 @@ class fdqExperiment:
             return
 
         remove_file(self.checkpoint_path)
-        self.checkpoint_path = os.path.join(
-            self.results_dir, f"checkpoint_e{self.current_epoch}.fdqcpt"
-        )
+        self.checkpoint_path = os.path.join(self.results_dir, f"checkpoint_e{self.current_epoch}.fdqcpt")
 
         iprint(f"Saving checkpoint to {self.checkpoint_path}")
 
@@ -735,9 +685,7 @@ class fdqExperiment:
 
         torch.save(checkpoint, self.checkpoint_path)
 
-    def get_next_export_fn(
-        self, name: str | None = None, file_ending: str = "jpg"
-    ) -> str:
+    def get_next_export_fn(self, name: str | None = None, file_ending: str = "jpg") -> str:
         if not self.is_main_process():
             return None
 
@@ -749,9 +697,7 @@ class fdqExperiment:
             dest_dir = self.results_output_dir
 
         name_str = "" if name is None else f"__{name}"
-        path = os.path.join(
-            dest_dir, f"{start_str}_{self.file_store_cnt:02}{name_str}.{file_ending}"
-        )
+        path = os.path.join(dest_dir, f"{start_str}_{self.file_store_cnt:02}{name_str}.{file_ending}")
 
         self.file_store_cnt += 1
 
@@ -854,7 +800,26 @@ class fdqExperiment:
                     optimizer.step()
                 optimizer.zero_grad()
 
-    def finalize_epoch(
+    def on_epoch_start(self, epoch: int) -> None:
+        if epoch is None:
+            self.current_epoch += 1
+        else:
+            self.current_epoch = epoch
+
+        self.current_ep_start_time = datetime.now()
+
+        iprint(f"\nEpoch: {self.current_epoch + 1} / {self.nb_epochs}")
+
+        if self.is_distributed():
+            # necessary to make shuffling work properly
+            for data_name, _ in self.exp_def.data.items():
+                if self.data[data_name] is not None:
+                    if self.data[data_name].train_sampler is not None:
+                        self.data[data_name].train_sampler.set_epoch(epoch)
+                    if self.data[data_name].val_sampler is not None:
+                        self.data[data_name].val_sampler.set_epoch(epoch)
+
+    def on_epoch_end(
         self,
         log_scalars: dict[str, float] | None = None,
         log_images_wandb: Any | None = None,
@@ -887,19 +852,23 @@ class fdqExperiment:
             scalars=log_scalars,
         )
 
-        # end of last epoch
+        try:
+            current_ep_time = datetime.now() - self.current_ep_start_time
+            self.total_run_time = datetime.now() - self.creation_time
+
+            iprint(
+                f"Total run time: {self.total_run_time.days} days, "
+                f"{self.total_run_time.seconds // 3600} hours, "
+                f"{self.total_run_time.seconds % 3600 / 60.0:.0f} minutes | "
+                f"current epoch: {int(current_ep_time.total_seconds() // 60)} minutes {int(current_ep_time.total_seconds() % 60)} seconds"
+            )
+        except (AttributeError, ValueError, TypeError):
+            iprint("Error calculating epoch time - skipping.")
+
+        store_processing_infos(self)
+
         if self.current_epoch == self.nb_epochs - 1:
             self.finish_time = datetime.now()
-            store_processing_infos(self)
-
-        try:
-            self.run_time = datetime.now() - self.creation_time
-            td = self.run_time
-            run_t_string = f"days: {td.days}, hours: {td.seconds // 3600}, minutes: {td.seconds % 3600 / 60.0:.0f}"
-            iprint(f"Current run time: {run_t_string}")
-            store_processing_infos(self)
-        except (AttributeError, ValueError):
-            pass
 
         save_train_history(self)
         self.save_checkpoint()
@@ -938,19 +907,21 @@ class fdqExperiment:
             return
 
         if self.is_main_process():
-            if not os.path.exists(self.scratch_data_path):
-                os.makedirs(self.scratch_data_path)
+            os.makedirs(self.scratch_data_path, exist_ok=True)
 
-            iprint("----------------------------------------------------")
-            iprint("Copy datasets to temporary scratch location...")
-            iprint("----------------------------------------------------")
+        iprint("-----------------------------------------------------------")
+        iprint("Copy datasets to temporary scratch location...")
+        iprint("-----------------------------------------------------------")
 
-            for data_name, data_source in self.exp_def.data.items():
-                dargs = data_source.args
+        for data_name, data_source in self.exp_def.data.items():
+            dargs = data_source.args
 
-                if dargs.base_path is not None:
+            if dargs.base_path is not None:
+                dst_path = os.path.join(self.scratch_data_path, data_name)
+
+                # actual actions only on rank 0
+                if self.is_main_process():
                     # cleanup old data first -> in case this is a debug run with dirty data
-                    dst_path = os.path.join(self.scratch_data_path, data_name)
                     if os.path.exists(dst_path):
                         shutil.rmtree(dst_path)
 
@@ -966,9 +937,7 @@ class fdqExperiment:
                                 text=True,
                                 check=True,
                             )
-                            iprint(
-                                f"Successfully copied {dargs.base_path} to {dst_path}"
-                            )
+                            iprint(f"Successfully copied {dargs.base_path} to {dst_path}")
 
                         except Exception as exc:
                             raise ValueError(
@@ -976,60 +945,47 @@ class fdqExperiment:
                                 f"Return code: {exc.returncode}, Error: {exc.stderr}"
                             ) from exc
 
-                    dargs.base_path = dst_path
-                else:
-                    for file_cat in [
-                        "train_files_path",
-                        "test_files_path",
-                        "val_files_path",
-                    ]:
-                        fps = dargs.get(file_cat)
-                        if not fps:
-                            continue
-                        if not isinstance(fps, list):
-                            raise ValueError(
-                                f"Error, {data_name} dataset files must be a list of file paths. Got: {fps}"
-                            )
-                        new_paths = []
-                        pbar = startProgBar(
-                            len(fps), f"Copy {file_cat} files to scratch"
-                        )
-                        for i, src_file in enumerate(fps):
-                            pbar.update(i)
-                            rel_path = os.path.relpath(src_file, "/")
-                            dst_file = os.path.join(self.scratch_data_path, rel_path)
+                dargs.base_path = dst_path
+
+            else:
+                for file_cat in [
+                    "train_files_path",
+                    "test_files_path",
+                    "val_files_path",
+                ]:
+                    fps = dargs.get(file_cat)
+                    if not fps:
+                        continue
+                    if not isinstance(fps, list):
+                        raise ValueError(f"Error, {data_name} dataset files must be a list of file paths. Got: {fps}")
+                    new_paths = []
+                    pbar = startProgBar(len(fps), f"Copy {file_cat} files to scratch")
+                    for i, src_file in enumerate(fps):
+                        pbar.update(i)
+                        rel_path = os.path.relpath(src_file, "/")
+                        dst_file = os.path.join(self.scratch_data_path, rel_path)
+                        if self.is_main_process():
                             os.makedirs(os.path.dirname(dst_file), exist_ok=True)
                             shutil.copy(src_file, dst_file)
-                            new_paths.append(dst_file)
-                        dargs.set(file_cat, new_paths)
-                        pbar.finish()
+                        new_paths.append(dst_file)
+                    dargs.set(file_cat, new_paths)
+                    pbar.finish()
+
+        iprint("-----------------------------------------------------------")
+        iprint("Copy datasets to temporary scratch location... Done!")
+        iprint("-----------------------------------------------------------")
 
-            iprint("----------------------------------------------------")
-            iprint("Copy datasets to temporary scratch location... Done!")
-            iprint("----------------------------------------------------")
         self.dist_barrier()
 
-    def dist_barrier(self) -> None:
-        """Barrier for distributed training."""
-        if self.is_distributed():
-            torch.distributed.barrier()
-
     def prepare_transformers(self) -> None:
         """Prepare transformers for the experiment."""
         if self.exp_def.transforms is None:
             return
         if not isinstance(self.exp_def.to_dict().get("transforms"), dict):
-            raise ValueError(
-                "Error, transforms must be a dictionary with transform names as keys."
-            )
+            raise ValueError("Error, transforms must be a dictionary with transform names as keys.")
 
         for transformer_name, transformer_def in self.exp_def.transforms.items():
-            self.transformers[transformer_name] = get_transformers(
-                t_defs=transformer_def
-            )
-
-    def dump_model(self) -> None:
-        dump_model(self)
+            self.transformers[transformer_name] = get_transformers(t_defs=transformer_def)
 
     def print_model(self) -> None:
         if not self.is_main_process():
@@ -1048,9 +1004,7 @@ class fdqExperiment:
             iprint("-----------------------------------------------------------\n")
 
             try:
-                iprint(
-                    f"Saving model graph to: {self.results_dir}/{model_name}_graph.png"
-                )
+                iprint(f"Saving model graph to: {self.results_dir}/{model_name}_graph.png")
 
                 sample = next(iter(self.data[next(iter(self.data))].train_data_loader))
                 if isinstance(sample, tuple):
