@@ -10,6 +10,8 @@ from fdq.experiment import fdqExperiment
 from fdq.testing import run_test
 from fdq.ui_functions import iprint
 from fdq.misc import load_conf_file
+from fdq.dump import dump_model
+from fdq.inference import inference_model
 
 
 def parse_args() -> argparse.Namespace:
@@ -20,15 +22,9 @@ def parse_args() -> argparse.Namespace:
     argparse.Namespace
         Parsed command-line arguments.
     """
-    parser: argparse.ArgumentParser = argparse.ArgumentParser(
-        description="FCQ deep learning framework."
-    )
-    parser.add_argument(
-        "experimentfile", type=str, help="Path to experiment definition file."
-    )
-    parser.add_argument(
-        "-nt", "-notrain", dest="train_model", default=True, action="store_false"
-    )
+    parser: argparse.ArgumentParser = argparse.ArgumentParser(description="FCQ deep learning framework.")
+    parser.add_argument("experimentfile", type=str, help="Path to experiment definition file.")
+    parser.add_argument("-nt", "-notrain", dest="train_model", default=True, action="store_false")
     parser.add_argument(
         "-ti",
         "-test_interactive",
@@ -43,13 +39,9 @@ def parse_args() -> argparse.Namespace:
         default=False,
         action="store_true",
     )
-
-    parser.add_argument(
-        "-d", "-dump", dest="dump_model", default=False, action="store_true"
-    )
-    parser.add_argument(
-        "-p", "-printmodel", dest="print_model", default=False, action="store_true"
-    )
+    parser.add_argument("-d", "-dump", dest="dump_model", default=False, action="store_true")
+    parser.add_argument("-i", "-inference", dest="inference_model", default=False, action="store_true")
+    parser.add_argument("-p", "-printmodel", dest="print_model", default=False, action="store_true")
     parser.add_argument(
         "-rp",
         "-resume_path",
@@ -87,7 +79,10 @@ def start(rank: int, args: argparse.Namespace) -> None:
         run_test(experiment)
 
     if experiment.inargs.dump_model:
-        experiment.dump_model()
+        dump_model(experiment)
+
+    if experiment.inargs.inference_model:
+        inference_model(experiment)
 
     experiment.clean_up_distributed()
 
@@ -97,9 +92,7 @@ def start(rank: int, args: argparse.Namespace) -> None:
     # if NaN or very early stop detected
     if experiment.early_stop_reason == "NaN_train_Loss":
         sys.exit(1)
-    elif experiment.early_stop_detected and experiment.current_epoch < int(
-        0.1 * experiment.nb_epochs
-    ):
+    elif experiment.early_stop_detected and experiment.current_epoch < int(0.1 * experiment.nb_epochs):
         sys.exit(1)
 
 
@@ -108,18 +101,12 @@ def main():
     inargs = parse_args()
 
     if inargs.train_model:
-        world_size = (
-            load_conf_file(inargs.experimentfile)
-            .get("slurm_cluster", {})
-            .get("world_size", 1)
-        )
+        world_size = load_conf_file(inargs.experimentfile).get("slurm_cluster", {}).get("world_size", 1)
     else:
         world_size = 1
 
     if world_size > torch.cuda.device_count():
-        raise ValueError(
-            f"ERROR, world size {world_size} is larger than available GPUs: {torch.cuda.device_count()}"
-        )
+        raise ValueError(f"ERROR, world size {world_size} is larger than available GPUs: {torch.cuda.device_count()}")
 
     if world_size == 1:
         # No need for multiprocessing
