@@ -9,6 +9,11 @@ class AddValueTransform:
     """A transform that adds a specified value to the input tensor."""
 
     def __init__(self, value):
+        """Initialize the AddValueTransform.
+
+        Args:
+            value: The value to add to the input tensor.
+        """
         self.value = value
 
     def __call__(self, t):
@@ -19,6 +24,11 @@ class MultValueTransform:
     """A transform that multiplies the input tensor by a specified value."""
 
     def __init__(self, value):
+        """Initialize the MultValueTransform.
+
+        Args:
+            value: The value to multiply the input tensor by.
+        """
         self.value = value
 
     def __call__(self, t):
@@ -29,6 +39,11 @@ class DivValueTransform:
     """A transform that divides the input tensor by a specified value."""
 
     def __init__(self, value):
+        """Initialize the DivValueTransform.
+
+        Args:
+            value: The value to divide the input tensor by.
+        """
         self.value = value
 
     def __call__(self, t):
@@ -39,6 +54,12 @@ class ClampAbsTransform:
     """A transform that clamps the input tensor to the specified lower and upper bounds."""
 
     def __init__(self, lower, upper):
+        """Initialize the ClampAbsTransform.
+
+        Args:
+            lower: The lower bound for clamping.
+            upper: The upper bound for clamping.
+        """
         self.lower = lower
         self.upper = upper
 
@@ -55,6 +76,12 @@ class ClampPercTransform:
     """
 
     def __init__(self, lower_perc, upper_perc):
+        """Initialize the ClampPercTransform.
+
+        Args:
+            lower_perc (float): The lower percentile for clamping.
+            upper_perc (float): The upper percentile for clamping.
+        """
         self.lower_perc = lower_perc
         self.upper_perc = upper_perc
 
@@ -80,30 +107,40 @@ class ReRangeTransform:
     """Re-ranges the input tensor from [in_min, in_max] to [out_min, out_max]."""
 
     def __init__(self, in_min, in_max, out_min, out_max):
+        """Initialize the ReRangeTransform.
+
+        Args:
+            in_min: Minimum value of the input range.
+            in_max: Maximum value of the input range.
+            out_min: Minimum value of the output range.
+            out_max: Maximum value of the output range.
+        """
         self.in_min = in_min
         self.in_max = in_max
         self.out_min = out_min
         self.out_max = out_max
 
     def __call__(self, t):
-        return (t - self.in_min) / (self.in_max - self.in_min) * (
-            self.out_max - self.out_min
-        ) + self.out_min
+        return (t - self.in_min) / (self.in_max - self.in_min) * (self.out_max - self.out_min) + self.out_min
 
 
 class ReRangeMinMaxTransform:
     """Re-ranges the input tensor so that its minimum and maximum values are mapped to out_min and out_max, respectively."""
 
     def __init__(self, out_min, out_max):
+        """Initialize the ReRangeMinMaxTransform.
+
+        Args:
+            out_min: Minimum value of the output range.
+            out_max: Maximum value of the output range.
+        """
         self.out_min = out_min
         self.out_max = out_max
 
     def __call__(self, t):
         t_min = t.min()
         t_max = t.max()
-        return (t - t_min) / (t_max - t_min) * (
-            self.out_max - self.out_min
-        ) + self.out_min
+        return (t - t_min) / (t_max - t_min) * (self.out_max - self.out_min) + self.out_min
 
 
 class Stack3DTransform:
@@ -114,6 +151,11 @@ class Stack3DTransform:
     """
 
     def __init__(self, stack_n):
+        """Initialize the Stack3DTransform.
+
+        Args:
+            stack_n (int): The number of times to stack the input tensor along the new dimension.
+        """
         self.stack_n = stack_n
 
     def __call__(self, t):
@@ -143,6 +185,14 @@ class ResizeMaxDimPadTransform:
     """
 
     def __init__(self, max_dim, interpol_mode="bilinear", mode="constant", value=0):
+        """Initialize the ResizeMaxDimPadTransform.
+
+        Args:
+            max_dim (int): The maximum dimension for resizing.
+            interpol_mode (str): Interpolation mode for resizing (e.g., 'bilinear').
+            mode (str): Padding mode ('constant', 'edge', 'replicate', or 'circular').
+            value (int or float): Fill value for 'constant' padding.
+        """
         self.max_dim = max_dim
         self.interpol_mode = interpol_mode
         self.mode = mode
@@ -171,9 +221,7 @@ class ResizeMaxDimPadTransform:
         pad_left = pad_w // 2
         pad_right = pad_w - pad_left
 
-        return F.pad(
-            t, (pad_left, pad_right, pad_top, pad_bottom), self.mode, self.value
-        )
+        return F.pad(t, (pad_left, pad_right, pad_top, pad_bottom), self.mode, self.value)
 
 
 class PaddingTransform:
@@ -193,6 +241,13 @@ class PaddingTransform:
     """
 
     def __init__(self, padding_size, padding_mode="constant", padding_value=0):
+        """Initialize the PaddingTransform.
+
+        Args:
+            padding_size: Padding values (tuple).
+            padding_mode (str): Padding mode ('constant', 'edge', 'replicate', or 'circular').
+            padding_value: Fill value for 'constant' padding.
+        """
         if padding_mode not in ["constant", "edge", "replicate", "circular"]:
             raise ValueError(f"Padding mode {padding_mode} not supported!")
 
@@ -217,36 +272,34 @@ class UnPaddingTransform:
     """
 
     def __init__(self, padding_size):
+        """Initialize the UnPaddingTransform.
+
+        Args:
+            padding_size: Padding values (tuple) - same format as used in PaddingTransform.
+        """
         self.padding_size = padding_size
 
     def __call__(self, t):
-        req_pad_dim = t.dim() * 2
-        pad = self.padding_size + [0] * (req_pad_dim - len(self.padding_size))
-        ps_start = pad[::2]
-        ps_stop = pad[1::2]
+        pad_spec = list(self.padding_size)
+        if len(pad_spec) % 2 != 0:
+            raise ValueError("Padding specification must have an even number of elements.")
 
-        for dim in range(t.dim()):
-            if ps_stop[dim] == 0:
-                ps_stop[dim] = t.shape[dim]
-            else:
-                ps_stop[dim] = t.shape[dim] - ps_stop[dim]
+        num_padded_dims = len(pad_spec) // 2
+        if num_padded_dims > t.dim():
+            raise ValueError(f"Padding specification for {num_padded_dims} dims exceeds tensor dims {t.dim()}")
 
-        if t.dim() == 4:
-            return t[
-                ps_start[0] : ps_stop[0],
-                ps_start[1] : ps_stop[1],
-                ps_start[2] : ps_stop[2],
-                ps_start[3] : ps_stop[3],
-            ]
-        if t.dim() == 5:
-            return t[
-                ps_start[0] : ps_stop[0],
-                ps_start[1] : ps_stop[1],
-                ps_start[2] : ps_stop[2],
-                ps_start[3] : ps_stop[3],
-                ps_start[4] : ps_stop[4],
-            ]
-        raise ValueError("Only 4D and 5D tensors are supported!")
+        slices = [slice(None)] * t.dim()
+        # PyTorch F.pad order: (pad_left, pad_right, pad_top, pad_bottom, ...), i.e. last dim first.
+        for i in range(num_padded_dims):
+            left = pad_spec[2 * i]
+            right = pad_spec[2 * i + 1]
+            dim = t.dim() - 1 - i  # map i-th pair to trailing dimension
+            start = left if left > 0 else 0
+            # If right == 0 we leave end as None to take full extent
+            end = -right if right > 0 else None
+            slices[dim] = slice(start, end)
+
+        return t[tuple(slices)]
 
 
 class Float32Transform:
@@ -278,6 +331,12 @@ class Get2DFrom3DTransform:
     """
 
     def __init__(self, axis, index):
+        """Initialize the Get2DFrom3DTransform.
+
+        Args:
+            axis (int): The axis along which to select the slice.
+            index (int): The index of the slice to select.
+        """
         self.axis = axis
         self.index = index
 
@@ -285,12 +344,61 @@ class Get2DFrom3DTransform:
         if self.index is None:
             self.index = t.shape[self.axis] // 2  # Default to the middle slice
         if self.axis < 0 or self.axis >= t.dim():
-            raise ValueError(
-                f"Axis {self.axis} is out of bounds for the tensor with {t.dim()} dimensions."
-            )
+            raise ValueError(f"Axis {self.axis} is out of bounds for the tensor with {t.dim()} dimensions.")
         return t.select(dim=self.axis, index=self.index)
 
 
+class SynchronizedRandomVerticalFlip:
+    """Apply synchronized random vertical flip (upside down along the vertical axis) to multiple tensors."""
+
+    def __init__(self, p=0.5, generator=None):
+        """Initialize the SynchronizedRandomVerticalFlip transform.
+
+        Args:
+            p (float): Probability of applying the flip. Default is 0.5.
+            generator (torch.Generator, optional): Random number generator for deterministic behavior.
+                       Allows consistent random decision across distributed processes.
+        """
+        self.p = p
+        self.generator = generator
+
+    def __call__(self, *tensors):
+        if torch.rand((), generator=self.generator).item() < self.p:
+            return tuple(torch.flip(tensor, dims=[-2]) for tensor in tensors)
+        return tensors
+
+    def apply_transform(self, tensors):
+        """Apply the vertical flip transform to tensors.
+
+        Args:
+            tensors: Input tensors to transform.
+
+        Returns:
+            Transformed tensors.
+        """
+        return self(*tensors)
+
+
+class SynchronizedRandomHorizontalFlip:
+    """Apply synchronized random horizontal flip (left-right along the horizontal axis) to multiple tensors."""
+
+    def __init__(self, p=0.5, generator=None):
+        """Initialize the SynchronizedRandomHorizontalFlip transform.
+
+        Args:
+            p (float): Probability of applying the flip. Default is 0.5.
+            generator (torch.Generator, optional): Random number generator for deterministic behavior.
+                       Allows consistent random decision across distributed processes.
+        """
+        self.p = p
+        self.generator = generator
+
+    def __call__(self, *tensors):
+        if torch.rand((), generator=self.generator).item() < self.p:
+            return tuple(torch.flip(tensor, dims=[-1]) for tensor in tensors)
+        return tensors
+
+
 def get_transformers(t_defs: Any) -> transforms.Compose:
     """Compose a sequence of transformers specified by their names or configuration dictionaries.
 
@@ -303,9 +411,7 @@ def get_transformers(t_defs: Any) -> transforms.Compose:
     return transforms.Compose([get_transformer(t) for t in t_defs])
 
 
-def get_transformer_by_names(
-    transformer_name: str, parameters: dict[str, Any] | None, t_defs: Any
-) -> Callable:
+def get_transformer_by_names(transformer_name: str, parameters: dict[str, Any] | None, t_defs: Any) -> Callable:
     """Returns a torchvision transformer instance based on the given transformer name and parameters.
 
     Args:
@@ -320,9 +426,7 @@ def get_transformer_by_names(
         transformer = Stack3DTransform(parameters["stack_n"])
 
     elif transformer_name == "Resize_HW":
-        transformer = transforms.Resize(
-            (parameters.get("h"), parameters.get("w")), antialias=True
-        )
+        transformer = transforms.Resize((parameters.get("h"), parameters.get("w")), antialias=True)
 
     elif transformer_name == "ResizeMaxDimPad":
         transformer = ResizeMaxDimPadTransform(
@@ -336,9 +440,7 @@ def get_transformer_by_names(
         transformer = ClampAbsTransform(parameters["lower"], parameters["upper"])
 
     elif transformer_name == "CLAMP_perc":
-        transformer = ClampPercTransform(
-            parameters["lower_perc"], parameters["upper_perc"]
-        )
+        transformer = ClampPercTransform(parameters["lower_perc"], parameters["upper_perc"])
 
     elif transformer_name == "ReRange":
         transformer = ReRangeTransform(
@@ -349,9 +451,7 @@ def get_transformer_by_names(
         )
 
     elif transformer_name == "ReRange_minmax":
-        transformer = ReRangeMinMaxTransform(
-            parameters["out_min"], parameters["out_max"]
-        )
+        transformer = ReRangeMinMaxTransform(parameters["out_min"], parameters["out_max"])
 
     elif transformer_name == "Gaussian_Blur":
         transformer = transforms.GaussianBlur(
@@ -370,9 +470,7 @@ def get_transformer_by_names(
         transformer = UnPaddingTransform(parameters["padding_size"])
 
     elif transformer_name == "Get2DFrom3D":
-        transformer = Get2DFrom3DTransform(
-            parameters.get("axis", 0), parameters.get("index")
-        )
+        transformer = Get2DFrom3DTransform(parameters.get("axis", 0), parameters.get("index"))
 
     elif transformer_name == "ADD":
         transformer = AddValueTransform(parameters["value"])
@@ -381,9 +479,7 @@ def get_transformer_by_names(
         transformer = DivValueTransform(parameters["value"])
 
     elif transformer_name == "NORM":
-        transformer = transforms.Normalize(
-            mean=(parameters.get("mean"),), std=(parameters.get("stdev"),)
-        )
+        transformer = transforms.Normalize(mean=(parameters.get("mean"),), std=(parameters.get("stdev"),))
 
     elif transformer_name == "MULT":
         transformer = MultValueTransform(parameters["value"])
@@ -406,17 +502,20 @@ def get_transformer_by_names(
         )
 
     elif transformer_name == "RandomHorizontalFlip":
-        transformer = transforms.RandomHorizontalFlip(
-            p=0.5 if parameters is None else parameters.get("p", 0.5)
-        )
+        transformer = transforms.RandomHorizontalFlip(p=0.5 if parameters is None else parameters.get("p", 0.5))
+
+    elif transformer_name == "SynchronizedRandomHorizontalFlip":
+        transformer = SynchronizedRandomHorizontalFlip(p=0.5 if parameters is None else parameters.get("p", 0.5))
 
     elif transformer_name == "RandomVerticalFlip":
-        transformer = transforms.RandomVerticalFlip(
-            p=0.5 if parameters is None else parameters.get("p", 0.5)
-        )
+        transformer = transforms.RandomVerticalFlip(p=0.5 if parameters is None else parameters.get("p", 0.5))
+
+    elif transformer_name == "SynchronizedRandomVerticalFlip":
+        transformer = SynchronizedRandomVerticalFlip(p=0.5 if parameters is None else parameters.get("p", 0.5))
 
     elif transformer_name == "ToTensor":
-        transformer = transforms.ToTensor()
+        # transformer = transforms.ToTensor() # deprecated
+        transformer = transforms.Compose([transforms.ToImage(), transforms.ToDtype(torch.float32, scale=True)])
 
     elif transformer_name == "Float32":
         transformer = Float32Transform()
@@ -425,9 +524,7 @@ def get_transformer_by_names(
         transformer = Uint8Transform()
 
     elif transformer_name == "RGB_Normalize":
-        transformer = transforms.Normalize(
-            mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]
-        )
+        transformer = transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225])
 
     elif transformer_name == "RGB2GRAY":
         transformer = transforms.Grayscale(num_output_channels=1)
@@ -604,29 +701,21 @@ def get_transformer(t_defs: Any) -> Callable:
     if isinstance(t_defs, dict):
         keys = list(t_defs.keys())
         if len(keys) != 1:
-            raise ValueError(
-                f"Transformation {t_defs} does not correspond to the expected format!"
-            )
+            raise ValueError(f"Transformation {t_defs} does not correspond to the expected format!")
         transformer_name = keys[0]
         parameters = t_defs[transformer_name]
     elif isinstance(t_defs, str):
         transformer_name = t_defs
         parameters = None
     else:
-        raise ValueError(
-            f"Transformation {t_defs} does not correspond to the expected format!"
-        )
+        raise ValueError(f"Transformation {t_defs} does not correspond to the expected format!")
 
     # check if all required parameters are present and datatypes are correct
     for req_param, req_type in all_required_params.get(transformer_name, {}).items():
         if req_param not in parameters:
-            raise ValueError(
-                f"Parameter {req_param} is missing for transformation {transformer_name}."
-            )
+            raise ValueError(f"Parameter {req_param} is missing for transformation {transformer_name}.")
         if type(parameters[req_param]) not in req_type:
-            raise ValueError(
-                f"Parameter {req_param} for transformation {transformer_name} is not correct."
-            )
+            raise ValueError(f"Parameter {req_param} for transformation {transformer_name} is not correct.")
 
     transformer = get_transformer_by_names(transformer_name, parameters, t_defs)
 
